chips = {}

function chips.load(word)
   chips.snap_distance        = 10
   chips.tile_speed           = 500
   chips.tile_set             = newAlphabetTileSet('chips_tiles', template.guess_tray.image:getWidth(), template.guess_tray.image:getHeight())
   chips.x                    = template.guess_tray.x
   chips.chars_y              = template.chars_tray.y + 10
   chips.guess_y              = template.guess_tray.y
      
   chips.tiles = {}
   for i = 1, #word do
      table.insert(chips.tiles, {
            char = string.sub(word, i, i),
            x = love.math.random(love.window.getWidth()  - chips.tile_set.tile_width),
            y = love.math.random(love.window.getHeight() - chips.tile_set.tile_height),
            target_x = chips.x + ((i - 1) * chips.tile_set.tile_width),
            target_y = chips.chars_y,
            status = 'in_chars'
         }
      )
   end
   chips.tile_set.batch:setBufferSize(#word)
   
   chips.error_statement = {text = '', x = love.window.getWidth() / 2, y = 240, timer = 0}
   chips.backspace_key = { pressed = true, counter = 0 }
   chips.previous_word = nil
   chips.error_sound   = love.audio.newSource('sounds/error.wav')
   chips.move_sound    = love.audio.newSource('sounds/move.wav')
   chips.shuffle_sound = love.audio.newSource('sounds/shuffle.wav')
   chips.solved_sound  = love.audio.newSource('sounds/solved.wav')
   chips.shuffle()
end
function chips.update(dt)
   if chips.backspace_key.pressed == true then
      chips.backspace_key.counter = chips.backspace_key.counter + dt
      if chips.backspace_key.counter >= 0.5 then
         chips.backspace_key.pressed = false
         chips.backspace_key.counter = 0
         chips.clear()
      end
   end
   if chips.error_statement.timer > 0 then
      chips.error_statement.timer = chips.error_statement.timer - dt
      if chips.error_statement.timer <= 0 then
         chips.clearErrorStatement()
      end
   end
   for i, v in pairs(chips.tiles) do
      v.x = chips.moveTile(v.x, v.target_x, dt)
      v.y = chips.moveTile(v.y, v.target_y, dt)
   end
   chips.tile_set.update(chips.tiles)
end
function chips.draw()
   love.graphics.setColor(200, 0, 0)
   love.graphics.print(chips.error_statement.text, chips.error_statement.x, chips.error_statement.y)
   love.graphics.setColor(255, 255, 255)
   
   chips.tile_set.draw()
end
function chips.unload()
   chips.tile_set        = nil
   chips.snap_distance   = nil
   chips.tile_speed      = nil
   chips.tile_quads      = nil
   chips.tiles           = nil
   
   chips.error_statement = nil
   chips.backspace_key   = nil
   chips.error_sound     = nil
   chips.move_sound      = nil
   chips.shuffle_sound   = nil
   chips.solved_sound    = nil
end

function chips.keypressed(key)
   if key == 'return' then
      local counter = 0
      for _, v in pairs(chips.tiles) do
         if v.status == 'in_guess' then
            counter = counter + 1
         end
      end
      if counter == 0 then
         chips.previous()
      else
         chips.enter()
      end
   elseif key == ' ' then
      chips.shuffle()
   elseif key == 'backspace' then
      chips.backspace_key.pressed = true
   else
      local key_to_index = 0
      for i, v in pairs(chips.tiles) do
         if v.status == 'in_chars' and v.char == string.upper(key) then
            key_to_index = i
         end
      end
      
      if key_to_index > 0 then
         chips.toGuessTray(key_to_index)
      end
   end
end
function chips.keyreleased(key)
   if key == 'backspace' then
      if chips.backspace_key.pressed == true then
         chips.backspace_key.pressed = false
         chips.backspace_key.counter = 0
         chips.backspace()
      end
   end
end

function chips.mousepressed(x, y, button)
   if button == 'l' then
      for i, v in pairs(chips.tiles) do
         if x >= v.target_x and x <= v.target_x + chips.tile_set.tile_width and
            y >= v.target_y and y <= v.target_y + chips.tile_set.tile_height then
            if v.status == 'in_guess' then
               chips.toCharsTray(i)
            elseif v.status == 'in_chars' then
               chips.toGuessTray(i)
            end
         end
      end
   end
end

----------

function chips.backspace()
   local key_to_index = 0
   local target_found = false
   local a = #chips.tiles
   
   while target_found == false and a >= 1 do
      for i, v in pairs(chips.tiles) do
         if v.status == 'in_guess' and v.target_x == chips.x + ((a - 1) * chips.tile_set.tile_width) then
            target_found = true
            key_to_index = i
         end
      end
      a = a - 1
   end
   
   if key_to_index > 0 then
      chips.toCharsTray(key_to_index)
   end
end

function chips.clear()
   local counter = 0
   for _, v in pairs(chips.tiles) do
      if v.status == 'in_guess' then
         counter = counter + 1
      end
   end
   
   for i = 1, counter do
      chips.backspace()
   end
end

function chips.clearErrorStatement()
   chips.error_statement.timer = 0
   chips.error_statement.text = ''
   chips.error_statement.x = love.window.getWidth() / 2
end

function chips.displayErrorStatement(text)
   chips.clear()
   chips.error_statement.timer = 5
   chips.error_statement.text = text
   chips.error_statement.x = (love.window.getWidth() / 2) - (state_game.font:getWidth(text) / 2)
   chips.error_sound:stop()
   chips.error_sound:play()
end

function chips.enter()
   local entered_set = ''
   for i = 1, #chips.tiles do
      local char_found = false
      for _, v in pairs(chips.tiles) do
         if v.status == 'in_guess' and v.target_x == chips.x + ((i - 1) * chips.tile_set.tile_width) then
            entered_set = entered_set .. v.char
            char_found = true
         end
      end
      if char_found == false then
         entered_set = entered_set .. ' '
      end
   end
   entered_set = chips.stringTrim(entered_set)
   if #entered_set > 0 then
      chips.previous_word = entered_set
   end
   if #entered_set > 2 then
      local error_value = 1
      for _, v in pairs(words.list) do
         if v.word == entered_set then
            
            if v.solved == false then
               v.solved = true
               error_value = 0
               state_game.score = state_game.score + ((string.len(v.word) ^ 2) * 10)
               chips.clear()
               chips.clearErrorStatement()
               chips.solved_sound:stop()
               chips.solved_sound:play()
            else
               error_value = 2
            end
         end
      end
      if error_value == 1 then
         chips.displayErrorStatement(entered_set .. ' does not exist in my dictionary!')
      elseif error_value == 2 then
         chips.displayErrorStatement('You already guessed ' .. entered_set .. '!')
      end
   else
      chips.displayErrorStatement('Please whack words longer than 3 letters!')
   end
   if #entered_set > 0 then
      chips.previous_word = entered_set:gsub("%s+", '')
   end
end

function chips.moveTile(cl, tl, dt)
   if math.sqrt((cl - tl) ^ 2) <=chips.snap_distance then
      cl = tl
   elseif cl < tl then
      cl = cl + (chips.tile_speed * dt)
   elseif cl > tl then
      cl = cl - (chips.tile_speed * dt)
   end
   return cl
end

function chips.previous()
   chips.clear()
   if chips.previous_word then
      for a = 1, #chips.previous_word do
         local key_to_index = 0
         for i, v in pairs(chips.tiles) do
            if v.status == 'in_chars' and v.char == string.sub(chips.previous_word, a, a) then
               key_to_index = i
            end
         end
         chips.toGuessTray(key_to_index)
      end
   end
end

function chips.shuffle()
   local shuffle_indexes = {}
   for _, v in pairs(chips.tiles) do
      if v.status == 'in_chars' then
         table.insert(shuffle_indexes, v.target_x)
      end
   end
   
   for i, v in pairs(chips.tiles) do
      if v.status == 'in_chars' then
         local r = love.math.random(#shuffle_indexes)
         v.target_x = shuffle_indexes[r]
         table.remove(shuffle_indexes, r)
      end
   end
   chips.shuffle_sound:stop()
   chips.shuffle_sound:play()
end

function chips.toCharsTray(index)
   local new_target = 0
   local target_found = false
   local a = #chips.tiles
   while target_found == false and a >= 1 do
      target_found = true
      for i, v in pairs(chips.tiles) do
         if v.status == 'in_chars' and v.target_x == chips.x + ((a - 1) * chips.tile_set.tile_width) then
            target_found = false
         end
      end
      a = a - 1
   end
   
   if target_found == true then
      chips.tiles[index].target_x = chips.x + (a * chips.tile_set.tile_width)
      chips.tiles[index].target_y = chips.chars_y
      chips.tiles[index].status = 'in_chars'
      chips.move_sound:stop()
      chips.move_sound:play()
   end
end

function chips.toGuessTray(index)
   local new_target = 0
   local target_found = false
   local a = 1
   while target_found == false and a <= #chips.tiles do
      target_found = true
      for i, v in pairs(chips.tiles) do
         if v.status == 'in_guess' and v.target_x == chips.x + ((a - 1) * chips.tile_set.tile_width) then
            target_found = false
         end
      end
      a = a + 1
   end
   
   if target_found == true then
      chips.tiles[index].target_x = chips.x + ((a - 2) * chips.tile_set.tile_width)
      chips.tiles[index].target_y = chips.guess_y
      chips.tiles[index].status = 'in_guess'
      chips.move_sound:stop()
      chips.move_sound:play()
   end
end

function chips.stringTrim(s)
   return s:find'^%s*$' and '' or s:match'^%s*(.*%S)'
end