require 'scripts/dictionary'
require 'scripts/template'
require 'scripts/buttons'
require 'scripts/timer'
require 'scripts/words'
require 'scripts/chips'

state_game = state_manager.newState()
state_game.font = love.graphics.newFont(20)

function state_game.load()
   state_game.loadNewWord()
   state_game.score = 0
end
function state_game.update(dt)
   template.update()
   words.update()
   timer.update(dt)
   chips.update(dt)
end
function state_game.draw()
   template.draw()
   timer.draw()
   love.graphics.setColor(0, 0, 0)
   love.graphics.print('SCORE : ' .. state_game.score, 0, 570)
   love.graphics.setColor(255, 255, 255)
   buttons.draw()
   words.draw()
   chips.draw()
end
function state_game.unload()
   template.unload()
   timer.unload()
   buttons.unload()
   words.unload()
   chips.unload()
   
   state_game.score = nil
end

function state_game.keypressed(key)
   chips.keypressed(key)
end
function state_game.keyreleased(key)
   chips.keyreleased(key)
end

function state_game.mousepressed(x, y, button)
   chips.mousepressed(x, y, button)
   buttons.mousepressed(x, y, button)
end

--I don't know where else to put this function, so there

function state_game.alphabetTileQuads(tile_width, tile_height, image_width, image_height)
   temp_tile_quads = {}
   temp_tile_quads['A'] = love.graphics.newQuad(              0,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['B'] = love.graphics.newQuad(tile_width     ,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['C'] = love.graphics.newQuad(tile_width *  2,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['D'] = love.graphics.newQuad(tile_width *  3,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['E'] = love.graphics.newQuad(tile_width *  4,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['F'] = love.graphics.newQuad(tile_width *  5,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['G'] = love.graphics.newQuad(tile_width *  6,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['H'] = love.graphics.newQuad(tile_width *  7,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['I'] = love.graphics.newQuad(tile_width *  8,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['J'] = love.graphics.newQuad(tile_width *  9,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['K'] = love.graphics.newQuad(tile_width * 10,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['L'] = love.graphics.newQuad(tile_width * 11,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['M'] = love.graphics.newQuad(tile_width * 12,           0, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['N'] = love.graphics.newQuad(              0, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['O'] = love.graphics.newQuad(tile_width     , tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['P'] = love.graphics.newQuad(tile_width *  2, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['Q'] = love.graphics.newQuad(tile_width *  3, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['R'] = love.graphics.newQuad(tile_width *  4, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['S'] = love.graphics.newQuad(tile_width *  5, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['T'] = love.graphics.newQuad(tile_width *  6, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['U'] = love.graphics.newQuad(tile_width *  7, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['V'] = love.graphics.newQuad(tile_width *  8, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['W'] = love.graphics.newQuad(tile_width *  9, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['X'] = love.graphics.newQuad(tile_width * 10, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['Y'] = love.graphics.newQuad(tile_width * 11, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads['Z'] = love.graphics.newQuad(tile_width * 12, tile_height, tile_width, tile_height, image_width, image_height)
   temp_tile_quads[' '] = love.graphics.newQuad(tile_width * 13, tile_height, tile_width, tile_height, image_width, image_height)
   return temp_tile_quads
end

function state_game.loadNewWord()
   words.unload()
   template.unload()
   timer.unload()
   chips.unload()
   buttons.unload()
   
   local word = candidate_words[love.math.random(#candidate_words)]
   --local word = 'ZITHER' --For debugging

   words.load(word)
   template.load(word)
   timer.load()
   chips.load(word)
   buttons.load()
end