state_manager = {}
states = {}

function state_manager.load()
   states[#states].load()
end
function state_manager.update(dt)
   if #states > 0 then
      states[#states].update(dt)
   else
      love.event.quit()
   end
end
function state_manager.draw()
   if #states > 0 then
      states[#states].draw()
   end
end

function state_manager.keypressed(key)
   states[#states].keypressed(key)
end
function state_manager.keyreleased(key)
   states[#states].keyreleased(key)
end

function state_manager.mousepressed(x, y, button)
   states[#states].mousepressed(x, y, button)
end
function state_manager.mousereleased(x, y, button)
   states[#states].mousereleased(x, y, button)
end

function state_manager.push(new_state)
   table.insert(states, new_state)
   love.graphics.setFont(states[#states].font)
   states[#states].load()
end
function state_manager.pop()
   states[#states].unload()
   table.remove(states, #states)
   
   if #states > 0 then
      love.graphics.setFont(states[#states].font)
   end
end

-----

function state_manager.newExitDialog(message, width, height)
   local dialog = state_manager.newState()
   local title = 'EXIT'
   
   function dialog.load()
      dialog.font    = states[#states - 1].font
      dialog.x       = (love.window.getWidth()  / 2) - (width  / 2)
      dialog.y       = (love.window.getHeight() / 2) - (height / 2)
      dialog.width   = width
      dialog.height  = height

      dialog.title   = {
         text = title,
         border_x = dialog.x + 2,
         border_y = dialog.y + 2,
         border_width = dialog.width - 4,
         border_height = dialog.font:getHeight(title) + 4,
         x = (dialog.x + dialog.width / 2) - (dialog.font:getWidth(title) / 2),
         y = dialog.y + 4,
      }
      
      dialog.message = {
         text = message,
         border_x = dialog.title.border_x,
         border_y = dialog.title.border_y + dialog.title.border_height + 4,
         border_width = dialog.title.border_width,
         border_height = dialog.height - dialog.title.border_height - 8,
         x = dialog.title.border_x + 2,
         y = dialog.title.border_y + dialog.title.border_height + 6,
         limit_x = dialog.title.border_width - 2
      }
      
      dialog.yes_button = {
         text = 'YES',
         x = (dialog.x + dialog.width / 2) - dialog.font:getWidth('YES') - 10,
         y = dialog.message.border_y + dialog.message.border_height - dialog.font:getHeight('YES') - 8,
      }
      dialog.yes_button.border_x = dialog.yes_button.x - 5
      dialog.yes_button.border_y = dialog.yes_button.y - 5
      dialog.yes_button.border_width = dialog.font:getWidth('YES') + 10
      dialog.yes_button.border_height = dialog.font:getHeight('YES') + 10
      
      dialog.no_button = {
         text = 'NO',
         border_x = dialog.yes_button.border_x + dialog.yes_button.border_width + 5,
         border_y = dialog.yes_button.border_y,
         border_width = dialog.yes_button.border_width,
         border_height = dialog.yes_button.border_height
      }
      dialog.no_button.x = (dialog.no_button.border_x + (dialog.no_button.border_width / 2)) - (dialog.font:getWidth('NO') / 2)
      dialog.no_button.y = dialog.yes_button.y
            
      love.graphics.setFont(dialog.font)
   end
   function dialog.draw()
      love.graphics.setColor(0, 180, 200)
      love.graphics.rectangle('fill', dialog.x, dialog.y, dialog.width, dialog.height)
      
      love.graphics.setColor(0, 0, 100)
      love.graphics.rectangle('line', dialog.title.border_x, dialog.title.border_y, dialog.title.border_width, dialog.title.border_height)
      love.graphics.rectangle('line', dialog.message.border_x, dialog.message.border_y, dialog.message.border_width, dialog.message.border_height)

      love.graphics.setColor(0, 0, 200)
      love.graphics.rectangle('fill', dialog.yes_button.border_x, dialog.yes_button.border_y, dialog.yes_button.border_width, dialog.yes_button.border_height)
      love.graphics.rectangle('fill', dialog.no_button.border_x, dialog.no_button.border_y, dialog.no_button.border_width, dialog.no_button.border_height)

      love.graphics.setColor(255, 255, 255)
      love.graphics.print(dialog.title.text, dialog.title.x, dialog.title.y)
      love.graphics.printf(dialog.message.text, dialog.message.x, dialog.message.y, dialog.message.limit_x, 'center')
      love.graphics.print(dialog.yes_button.text, dialog.yes_button.x, dialog.yes_button.y)
      love.graphics.print(dialog.no_button.text, dialog.no_button.x, dialog.no_button.y)
   end
   function dialog.unload()
      dialog.title      = nil
      dialog.message    = nil
      dialog.yes_button = nil
      dialog.no_button  = nil
      dialog.width      = nil
      dialog.height     = nil
      dialog.x          = nil
      dialog.y          = nil
   end
   
   function dialog.mousepressed(x, y, button)
      if dialog.isYesPressed(x, y, button) then
         state_manager.pop()
         state_manager.pop()
      elseif dialog.isNoPressed(x, y, button) then
         state_manager.pop()
      end
   end
   
   function dialog.isYesPressed(x, y, button)
      if button == 'l' and
         x >= dialog.yes_button.border_x and x <= dialog.yes_button.border_x + dialog.yes_button.border_width and
         y >= dialog.yes_button.border_y and y <= dialog.yes_button.border_y + dialog.yes_button.border_height then
         return true
      else
         return false
      end
   end
   function dialog.isNoPressed(x, y, button)
      if button == 'l' and
         x >= dialog.no_button.border_x and x <= dialog.no_button.border_x + dialog.no_button.border_width and
         y >= dialog.no_button.border_y and y <= dialog.no_button.border_y + dialog.no_button.border_height then
         return true
      else
         return false
      end
   end
   
   return dialog
end

function state_manager.newOKDialog(title, message, width, height)
   local dialog = state_manager.newState()
   
   function dialog.load()
      dialog.font    = states[#states - 1].font
      dialog.x       = (love.window.getWidth()  / 2) - (width  / 2)
      dialog.y       = (love.window.getHeight() / 2) - (height / 2)
      dialog.width   = width
      dialog.height  = height

      dialog.title   = {
         text = title,
         border_x = dialog.x + 2,
         border_y = dialog.y + 2,
         border_width = dialog.width - 4,
         border_height = dialog.font:getHeight(title) + 4,
         x = (dialog.x + dialog.width / 2) - (dialog.font:getWidth(title) / 2),
         y = dialog.y + 4,
      }
      
      dialog.message = {
         text = message,
         border_x = dialog.title.border_x,
         border_y = dialog.title.border_y + dialog.title.border_height + 4,
         border_width = dialog.title.border_width,
         border_height = dialog.height - dialog.title.border_height - 8,
         x = dialog.title.border_x + 2,
         y = dialog.title.border_y + dialog.title.border_height + 6,
         limit_x = dialog.title.border_width - 2
      }
      
      dialog.button = {
         text = 'OK',
         x = (dialog.x + dialog.width / 2) - (dialog.font:getWidth('OK') / 2),
         y = dialog.message.border_y + dialog.message.border_height - dialog.font:getHeight('OK') - 8,
      }
      dialog.button.border_x = dialog.button.x - 5
      dialog.button.border_y = dialog.button.y - 5
      dialog.button.border_width = dialog.font:getWidth('OK') + 10
      dialog.button.border_height = dialog.font:getHeight('OK') + 10

      love.graphics.setFont(dialog.font)
   end
   function dialog.draw()
      if dialog.title.text ~= 'PAUSE' then
         states[#states - 1].draw()
      end
      
      love.graphics.setColor(0, 180, 200)
      love.graphics.rectangle('fill', dialog.x, dialog.y, dialog.width, dialog.height)
      
      love.graphics.setColor(0, 0, 100)
      love.graphics.rectangle('line', dialog.title.border_x, dialog.title.border_y, dialog.title.border_width, dialog.title.border_height)
      love.graphics.rectangle('line', dialog.message.border_x, dialog.message.border_y, dialog.message.border_width, dialog.message.border_height)

      love.graphics.setColor(0, 0, 200)
      love.graphics.rectangle('fill', dialog.button.border_x, dialog.button.border_y, dialog.button.border_width, dialog.button.border_height)

      love.graphics.setColor(255, 255, 255)
      love.graphics.print(dialog.title.text, dialog.title.x, dialog.title.y)
      love.graphics.printf(dialog.message.text, dialog.message.x, dialog.message.y, dialog.message.limit_x, 'center')
      love.graphics.print(dialog.button.text, dialog.button.x, dialog.button.y)
   end
   function dialog.unload()
      dialog.title   = nil
      dialog.message = nil
      dialog.button  = nil
      dialog.width   = nil
      dialog.height  = nil
      dialog.x       = nil
      dialog.y       = nil
   end
   
   function dialog.mousepressed(x, y, button)
      if dialog.isButtonPressed(x, y, button) then
         if dialog.title.text == 'GAME OVER' then
            state_manager.pop()
            state_manager.pop()
         elseif dialog.title.text == 'WORD WHACKED' then
            state_manager.pop()
            state_game.loadNewWord()
         else
            state_manager.pop()
         end
      end
   end
   
   function dialog.isButtonPressed(x, y, button)
      if button == 'l' and
         x >= dialog.button.border_x and x <= dialog.button.border_x + dialog.button.border_width and
         y >= dialog.button.border_y and y <= dialog.button.border_y + dialog.button.border_height then
         return true
      else
         return false
      end
   end
   
   return dialog
end

function state_manager.newState()
   local state_template = {
      font = love.graphics.newFont(12)
   }
   function state_template.load()                      end
   function state_template.update(dt)                  end
   function state_template.draw()                      end
   function state_template.unload()                    end
   function state_template.keypressed(key)             end
   function state_template.keyreleased(key)            end
   function state_template.mousepressed(x, y, button)  end
   function state_template.mousereleased(x, y, button) end
   
   return state_template
end